#!/usr/bin/perl
use Getopt::Long;
use MLDBM qw(DB_File Storable);
my %Files = (
    db_hash => '/etc/mail/spamassassin/FuzzyOcr.db',
    db_safe => '/etc/mail/spamassassin/FuzzyOcr.safe.db',
    );  

# defaults
my $cfgfile = "FuzzyOcr.cf";
my %App;
my @bin_utils = qw/pamfile ppmhist jpegtopnm giftopnm pngtopnm bmptopnm/;
foreach (@bin_utils) {
    $App{$_} = "/usr/bin/$_";
}

my $delete = 0;
my $verbose = 0;
GetOptions(
    'verbose' => \$verbose,
    'delete'  => \$delete,
);

unless (@ARGV) {
    print "Usage: fuzzy-find.pl [Options] (imagehash|imagefile) \n";
    print "\n";
    print "Available options:\n";
    print "--delete     Removes the hash from the database\n";
    print "--verbose    Show more informations\n";
    print "\n";
    exit 1;
}

# Read custom paths from FuzzyOcr.cf
open CONFIG, "< $cfgfile" or warn "Can't read configuration file, using defaults...\n";

while (<CONFIG>) {
    chomp;
    if ($_ =~ m/^focr_bin_(\w+) (.+)/) {
	$App{$1} = $2;
        printf "Found custom path \"$2\" for application \"$1\"\n" if $verbose
    }
}

close CONFIG;

while (@ARGV) {
    my $file = shift @ARGV;
    my @data = ();
    if ($file =~ m/(\d+):(\d+):(\d+):(\d+)/) {
        push @data, $1,$2,$3,$4;
    } elsif ($file eq ':::0') {
        $key = $file;
        $data[3] = 0;
    } else {
        next unless -r $file;
    }
    my $key = '';
    unless (@data) {
        my $app;
        if ($res =~ m/JPEG/) {
            $app = $App{jpegtopnm};
        } elsif ($res =~ m/PNG/) {
            $app = $App{pngtopnm};
        } elsif ($res =~ m/BMP/) {
            $app = $App{bmptopnm};
        } elsif ($res =~ m/PNM/) {
            $app = '/bin/cat';
        } else {
            $app = $App{giftopnm};
        }
        my @hist = `$app $file |$App{ppmhist} -noheader -`;
        my @res = `$app $file |$App{pamfile} -`;
        my ($h,$w) = (0,0);
        if ($res[0] =~ m/(\d+) by (\d+)/) {
            $w = $1; $h = $2;
            printf "Found ($h,$w)\n" if $verbose
        }
        my $c = scalar(@hist); my $cnt = 0;
        printf "Colors: %d\n",$c if $verbose;
        push @data, (stat($file))[7],$h,$w,$c;
        foreach (@hist) {
            $_ =~ s/ +/ /g;
            my @d = split(' ',$_);
            $hash .= sprintf("::%d:%d:%d:%d:%d",@d);
            last if ($cnt++ ge 5);
        }
        $key = substr($hash,2);
    }
    printf "Img = %9d %dx%dx%d\n",@data;
    printf "key = <$key>\n" if ($key);
    foreach my $ff (keys %Files) {
        my %DB;
        tie %DB, 'MLDBM', $Files{$ff} or next;
        printf "Searching $Files{$ff}...\n";
        foreach my $kk (keys %DB) {
            my $db = $DB{$kk};
            my @dd = split('::',$kk);
            shift @dd if ($dd[0] !~ m/:/);
            my $dd = join('::',@dd);
            if ($key eq '') {
                next unless ($db->{basic} eq join(':',@data));
            } else {
                next unless ($dd eq $key);
            }
            printf "%s HASH\n",($delete)?'Removing':'Found';
            if ($delete) {
                delete $DB{$kk};
            } else {
                printf "ImageInfo  : %9d:%d:%d:%d\n",split(':',$db->{basic});
                printf "Matched    : %4d Time(s)\n",$db->{match};
                printf "Calc.Score : %9.3f\n",$db->{score};
                printf "in DB since: %s\n",scalar(localtime($db->{input}));
                printf "Last Match : %s\n",scalar(localtime($db->{check}));
            }
        }
        untie %DB;
    }
}
